<?php

require_once("../MyCSV.class.php");
require_once("Apeform.class.php");

/*
 * LICENSE
 * 1. If you like to use this class for personal purposes, it's free.
 * 2. For comercial purposes, please contact me (http://maettig.com/email).
 *    I'll send a license to you.
 * 3. When you copy the framework you must copy this notice with the source
 *    code. You may alter the source code, but you have to put the original
 *    with your altered version.
 * 4. The license is for all files included in this bundle.
 *
 * KNOWN BUGS/TODO
 * - Catch 0x00 when uploading a file!
 * - Use PHP types only (null/bool/int/float/string)! Add "attribute": "signed",
 *   "unsigned", "binary".
 * - Table files aren't portable to older versions or external software when
 *   using PHP >=4.3.2 because fgetcsv() needs escaped backslashes then.
 * - Foreign key support should be possible, I think. If field is "xxx_id",
 *   open table "xxx" or "xxxs" (if exists), use "id" and display first field
 *   (and the ID!) value after "id". Don't forget to offer "override" in this
 *   case.
 */

/**
 * Universal administration tool and demo application for the TM::MyCSV class.
 *
 * Don't hesitate to
 * <a href="http://bugs.maettig.com/">report bugs or feature requests</a>.
 *
 * @author Thiemo Mttig (http://maettig.com/)
 * @version 2003-11-11
 * @package TM
 */
class MyCSVAdmin
{
    /**
     * @var string $SELF Represents the $_SERVER['PHP_SELF'] value.
     */
    var $SELF = "./";

    /**
     * @var string $dir Relative path where the .csv and .txt files are stored.
     */
    var $dir = "./";

    /**
     * Add the values in the following table to grand or deny access to specific
     * functions of the administrative interface. Default is -1, everything
     * enabled.
     *
     * - 2 - Insert rows
     * - 4 - Update rows
     * - 8 - Delete rows
     * - 32 - Alter tables
     * - 64 - Create tables
     * - 128 - Drop tables
     * - 8192 - Export tables</pre>
     *
     * Privilege 1 (select rows), 16 (create/alter/drop indices) and 256 to 4096
     * (server administration) aren't used.
     *
     * @var int $priv Privileges to grand or deny access to specific functions.
     */
    var $priv = -1;

    /**
     * @var MyCSV $table MyCSV object currently edited by TM::MyCSVAdmin.
     */
    var $table = null;

    /**
     * @var string $types Contains the suggested field types for the table.
     */
    var $types = array();

    /**
     * Initializes the class. Loads the table specified by $_GET['table'] or
     * $_POST['table']. Returns a new MyCSVAdmin object.
     *
     * @return MyCSVAdmin
     */
    function MyCSVAdmin()
    {
        if (isset($_SERVER['PHP_SELF'])) $this->SELF = $_SERVER['PHP_SELF'];
        elseif (isset($GLOBALS['PHP_SELF'])) $this->SELF = $GLOBALS['PHP_SELF'];
        $this->SELF = str_replace("index.php", "", $this->SELF);

        if ($this->GET('table'))
        {
            $this->table = new MyCSV($this->GET('table'));
            $this->_getTypes();
        }
    }

    /**
     * Gets a GET or POST value. This unifies the different behaviours of $_GET,
     * $HTTP_GET_VARS, $GLOBALS (register_globals) and so on.
     *
     * @param string $key Name of the request variable to be returned.
     * @return string
     */
    function GET($key)
    {
        if (isset($_GET[$key])) $value = $_GET[$key];
        elseif (isset($GLOBALS['HTTP_GET_VARS'][$key])) $value = $GLOBALS['HTTP_GET_VARS'][$key];
        elseif (isset($_POST[$key])) $value = $_POST[$key];
        elseif (isset($GLOBALS['HTTP_POST_VARS'][$key])) $value = $GLOBALS['HTTP_POST_VARS'][$key];
        else return "";
        if (get_magic_quotes_gpc()) return stripslashes($value);
        return $value;
    }

    /**
     * Makes a string well readable to the browser. It strips any critical
     * characters, truncates the string and replaces HTML entities.
     *
     * @param string $text Text to be HTMLed.
     * @return string
     */
    function htmlQuote($text)
    {
        if (strlen($text) > 100) $text = substr($text, 0, 80) . "...";
        $text = preg_replace('/[\x00-\x08\x0B-\x0C\x0E-\x1F\x7F\x81\x83\x86-\x8F\x90\x98-\x9F]/', '.', $text);
        return htmlentities($text);
    }

    /**
     * Tries to guess the field types based on what values are in the table
     * rows.
     *
     * @return void
     * @access private
     */
    function _getTypes()
    {
        $types = array(0 => "null", 1 => "bool", 2 => "int", 3 => "float", 4 => "varchar", 5 => "text", 6 => "blob");
        $tLeast = $this->table->count() < 2 ? 4 : 0;
        if ($this->GET('override')) { $types[1] = "int"; $types[4] = "text"; $types[6] = "text"; }
        foreach ($this->table->fields as $field)
        {
            $t[$field] = strcasecmp($field, "id") ? $tLeast : 2;
        }
        while ($row = $this->table->each())
        {
            foreach ($row as $field => $value)
            {
                if (preg_match('/[\x00-\x08\x0B-\x0C\x0E-\x13\x16-\x1F]/', $value))
                    $t[$field] = 6;
                elseif (strlen($value) > 255 || strpos($value, "\n"))
                    $t[$field] = max($t[$field], 5);
                elseif (! empty($value) && ! is_numeric($value))
                    $t[$field] = max($t[$field], 4);
                elseif (! preg_match('/^[+-]?\d{0,10}$/s', $value))
                    $t[$field] = max($t[$field], 3);
                elseif (! preg_match('/^[01]?$/s', $value))
                    $t[$field] = max($t[$field], 2);
                elseif (strlen($value))
                    $t[$field] = max($t[$field], 1);
            }
        }
        foreach ($this->table->fields as $field)
        {
            $this->types[$field] = $types[$t[$field]];
        }
        $this->table->reset();
    }

    /**
     * @return void
     * @access private
     */
    function displayHead()
    {
        echo '<html>';
        echo '<head>';
        echo '<title>TM::MyCSVAdmin</title>';
        echo '<meta name="robots" content="noindex,nofollow">';
        echo '<style type="text/css">';
        echo 'h1{font-size:150%}';
        echo '.subtitle{font-size:xx-small;font-weight:normal;}';
        echo 'ul#menu{border-bottom:2px solid #D2ECD2;margin-left:0;}';
        echo 'ul#menu li{background:#F4FAF4;border:1px solid #D2ECD2;display:inline;';
        echo 'padding:0px 10px;margin-left:10px;text-align:center;width:55px;}';
        echo 'th{background-color:#D2ECD2;}';
        echo 'td{background-color:#F4FAF4;}';
        echo 'small{color:#666666;}';
        echo 'u{color:#990000;font-weight:bold;}';
        echo '.danger:hover{background-color:#CC3300;color:white;text-decoration:none;}';
        echo '.download{text-decoration:none;}';
        echo '.override{text-decoration:none;}';
        echo 'a:hover{color:#CC3300;text-decoration:underline;}';
        echo '</style>';
        echo '</head>';
        echo '<body bgcolor="white" text="black" link="#006600" alink="#006600" vlink="#006600">';

        echo '<h1>';
        $script = preg_replace('{[^/]+(/index)?\.\w+$}', '', getenv('SCRIPT_NAME'));
        echo '<a class="subtitle" href="' . $script . '">Go to parrent Site</a><br>';
        echo 'TM::<a href="' . $this->SELF . '">MyCSVAdmin</a>';
        if (isset($this->table))
        {
            $table = $this->table->tablename();
            echo ' - Table <a href="' . $this->SELF . '?method=structure&table=' . $table . '">';
            echo '<em>' . $table . '</em></a>';
        }
        echo '</h1>';

        if (isset($this->table))
        {
            echo '<ul id="menu">';
            echo '<li><a href="' . $this->SELF . '?method=browse&table=' . $table . '">Browse</a></li>';
            if ($this->priv & 2)
                echo '<li><a href="' . $this->SELF . '?method=change&table=' . $table . '">Insert</a></li>';
            echo '<li><a href="' . $this->SELF . '?method=structure&table=' . $table . '">Structure</a></li>';
            if ($this->priv & 8192)
                echo '<li><a href="' . $this->SELF . '?method=export&table=' . $table . '">Export</a></li>';
            if ($this->priv & 128)
                echo '<li><a class="danger" href="' . $this->SELF . '?method=delete_all&table=' . $table . '">Empty</a></li>';
            if ($this->priv & 128)
                echo '<li><a class="danger" href="' . $this->SELF . '?method=drop_table&table=' . $table . '">Drop</a></li>';
            echo '</ul>';
        }
    }

    /**
     * Shows a list of all tables. Use {@link dir} to specify the working
     * directory.
     *
     * @return void
     */
    function tables()
    {
        $form = new Apeform(0, 0, false);
        $table = $form->text("Table Name");
        $form->submit("Create new Table");
        if ($form->isValid())
        {
            $this->table = new MyCSV($table);
            // Jump to the table structure.
            $this->structure();
            return;
        }
        
        $tables = array();
        $fp = opendir($this->dir);
        while (($filename = readdir($fp)) !== false)
        {
            $filename = $this->dir . $filename;
            if (! is_file($filename)) continue;
            if (! preg_match('/\.(csv|txt)$/i', $filename)) continue;
            $tables[] = $filename;
        }
        closedir($fp);
        usort($tables, 'strcasecmp');

        $this->displayHead();

        echo '<table>';
        echo '<tr><th>Table</th><th colspan="6">Action</th><th>Records</th><th>Size</th></tr>';
        $count = 0;
        $sumRecords = 0;
        $sumSize = 0;
        foreach ($tables as $filename)
        {
            $csv = new MyCSV($filename);
            $table = $csv->tablename();
            $count++;
            $sumRecords += $records = $csv->num_rows();
            $sumSize += $size = filesize($filename);
            $csv->close();

            echo '<tr><td>' . htmlspecialchars($table) . '</td>';
            echo '<td><a href="' . $this->SELF . '?method=browse&table=' . $table . '">Browse</a></td>';
            echo '<td>';
            if ($this->priv & 2)
                echo '<a href="' . $this->SELF . '?method=change&table=' . $table . '">Insert</a>';
            echo '</td>';
            echo '<td><a href="' . $this->SELF . '?method=structure&table=' . $table . '">Structure</a></td>';
            echo '<td>';
            if ($this->priv & 8192)
                echo '<a href="' . $this->SELF . '?method=export&table=' . $table . '">Export</a>';
            echo '</td><td>';
            if ($this->priv & 128)
                echo '<a class="danger" href="' . $this->SELF . '?method=delete_all&table=' . $table . '">Empty</a>';
            echo '</td><td>';
            if ($this->priv & 128)
                echo '<a class="danger" href="' . $this->SELF . '?method=drop_table&table=' . $table . '">Drop</a>';
            echo '</td>';
            echo '<td align="right">' . number_format($records) . '</td>';
            echo '<td align="right">' . number_format($size / 1024, 1) . ' KB</td>';
            echo '</tr>';
        }
        echo '<tr>';
        echo '<th>' . $count . ' Table(s)</th>';
        echo '<th colspan="6">Sum</th>';
        echo '<th align="right">' . number_format($sumRecords) . '</th>';
        echo '<th align="right">' . number_format($sumSize / 1024, 1) . ' KB</th>';
        echo '</tr>';
        echo '</table>';

        if ($this->priv & 64) $form->display();
    }
    
    /**
     * Shows the structure of a table.
     *
     * @return void
     */
    function structure()
    {
        $form = new Apeform(0, 0, false);
        $form->hidden("structure", "method");
        $form->hidden($this->table->tablename(), "table");
        $field = &$form->hidden($this->GET('method') == "structure" ? $this->GET('field') : "");
        $newField = &$form->text("Field Name", "", $field);

        foreach ($this->table->fields as $key) $positions[$key] = "After " . $key;
        $keys = array_keys($positions);
        $defaultValue = ($field && in_array($field, $keys)) ? $keys[array_search($field, $keys) - 1] : "";
        if ($field) unset($positions[$field]);
        array_pop($positions);
        $positions[''] = "At End of Table";
        $label = $field ? "Move to Position" : "Add at Position";
        $afterField = $form->select($label, "", $positions, $defaultValue);

        $button = $field ? "Rename/move Field" : "Add new Field";
        $form->submit($button);

        if ($form->isValid())
        {
            if (! $field)
            {
                if ($this->table->add_field($newField, $afterField)) $this->table->write();
                else $form->error("Invalid Field Name", -1);
            }
            else
            {
                while ($row = $this->table->each())
                {
                    // Copy all the moved values to their new field name.
                    $this->table->data[$row['id']][$newField] = $row[$field];
                }
                $newFields = array();
                foreach ($this->table->fields as $oldField)
                {
                    // Copy any unchanged field to the new fields array.
                    if ($oldField != $field) $newFields[] = $oldField;
                    if ($oldField == $afterField) $newFields[] = $newField;
                }
                // Add field if position '' ("At end of table") was selected.
                if (! $afterField) $newFields[] = $newField;
                $this->table->fields = $newFields;
                $this->table->write();
            }
            $field = "";
            $newField = "";
        }

        $this->displayHead();

        echo '<table>';
        echo '<tr><th>Field</th><th><small>Guessed Type</small></th><th colspan="2">Action</th></tr>';
        foreach ($this->table->fields as $i => $f)
        {
            echo '<tr><td>' . ($i ? $f : "<u>$f</u>") . '</td>';
            echo '<td><small>' . (isset($this->types[$f]) ? $this->types[$f] : "") . '</small></td>';
            echo '<td>';
            if ($i && $this->priv & 32)
                echo '<a href="' . $this->SELF . '?method=structure&table=' . $this->table->tablename() . '&field=' . $f . '">Change</a>';
            echo '</td><td>';
            if ($i && $this->priv & 32)
                echo '<a class="danger" href="' . $this->SELF . '?method=drop&table=' . $this->table->tablename() . '&field=' . $f . '">Drop</a>';
            echo '</td></tr>';
        }
        echo '</table>';

        if ($this->priv & 32) $form->display();

        echo '<p><table>';
        echo '<tr><th>Statements</th><th>Value</th></tr>';
        echo '<tr><td>Rows</td><td align="right">' . $this->table->num_rows() . '</td></tr>';
        echo '<tr><td>Next Autoindex</td><td align="right">' . ($this->table->num_rows() ? max($this->table->ids()) + 1 : 1) . '</td></tr>';
        $filesize = $this->table->exists() ? filesize($this->table->filename) : 0;
        $filemtime = $this->table->exists() ? date("Y-m-d H:i", filemtime($this->table->filename)) : "";
        echo '<tr><td>File Size</td><td align="right">' . number_format($filesize) . ' Bytes</td></tr>';
        echo '<tr><td>Average Row Size</td><td align="right">' . ($this->table->num_rows() ? round($filesize / $this->table->num_rows()) : 0) . ' Bytes</td></tr>';
        echo '<tr><td>Delimiter</td><td align="right">' . $this->table->delimiter . '</td></tr>';
        echo '<tr><td>Last Update</td><td align="right">' . $filemtime . '</td></tr>';
        echo '</table>';
    }

    /**
     * Shows the contents of a table.
     *
     * @return void
     */
    function browse()
    {
        $sort = $this->GET('sort');

        $this->displayHead();

        echo '<table>';
        echo '<tr><th colspan="2"></th>';
        foreach ($this->table->fields as $i => $field)
        {
            echo '<th><a href="' . $this->SELF;
            echo '?method=browse&table=' . $this->table->tablename() . '&sort=' . $field;
            if ($sort == $field) echo '%20DESC';
            $field = $this->htmlQuote($field);
            echo '" title="Sort">' . ($i ? $field : "<u>$field</u>") . '</a></th>';
        }
        echo '</tr>';

        if ($sort) $this->table->sort($sort . " SORT_NULL");
        $this->table->reset();
        while ($row = $this->table->each())
        {
            echo '<tr><td>';
            if ($this->priv & 4)
                echo '<a href="' . $this->SELF . '?method=change&table=' . $this->table->tablename() . '&id=' . urlencode($row['id']) . '">Edit</a>';
            echo '</td><td>';
            if ($this->priv & 8)
                echo '<a class="danger" href="' . $this->SELF . '?method=delete&table=' . $this->table->tablename() . '&id=' . urlencode($row['id']) . '">Delete</a>';
            echo '</td>';
            foreach ($this->table->fields as $field)
            {
                if (! isset($row[$field])) $row[$field] = "";
                switch ($this->types[$field])
                {
                    case "bool":
                        echo '<td align="center">' . ($row[$field] ? '&times;' : '-') . '</td>';
                        break;
                    case "int":
                    case "float":
                        echo '<td align="right">' . $row[$field] . '</td>';
                        break;
                    case "blob":
                        echo '<td><a href="' . $this->SELF;
                        echo '?method=download&table=' . $this->table->tablename();
                        echo '&id=' . urlencode($row['id']) . '&field=' . $field . '"';
                        echo ' class="download" title="Download Binary File">';
                        echo $this->htmlQuote($row[$field]) . '</a></td>';
                        break;
                    default:
                        echo '<td>' . $this->htmlQuote($row[$field]) . '</td>';
                }
            }
            echo '</tr>';
        }
        echo '</table>';
    
        if ($this->priv & 2)
            echo '<p><a href="' . $this->SELF . '?method=change&table=' . $this->table->tablename() . '">Insert new Row</a></p>';
    }
    
    /**
     * Shows a form to insert or update a row.
     *
     * @return void
     */
    function change()
    {
        $form = new Apeform(10000, 60, false);
        $form->hidden("change", "method");
        $table = $form->hidden($this->table->tablename(), "table");
        $id = $form->hidden($this->GET('id'), "id");
        $form->hidden($this->GET('override'), "override");
        $row = $this->table->data($id);
        foreach ($this->table->fields as $i => $field)
        {
            $label = $i ? $field : "<u>$field</u>";
            $help = "";
            if (! $this->GET('override'))
            {
                if ($field == "id") $help = "Primary Key";
                else $help = "Guessed Type: " . $this->types[$field];
                if ($this->types[$field] == "bool" || $this->types[$field] == "varchar" || $this->types[$field] == "blob")
                {
                    $help .= ' (<a class="override" href="' . $this->SELF . '?method=change';
                    $help .= '&table=' . $table . '&id=' . $id . '&override=true">override</a>)';
                }
            }
            $value = isset($row[$field]) ? $row[$field] : "";

            if (! $this->GET('override') && preg_match('/^(.+)_id$/i', $field, $matches))
            {
                $foreignTable = new MyCSV($matches[1]);
                if ($foreignTable->exists())
                {
                    $fField = $foreignTable->fields[1];
                    $foreignTable->sort($fField);
                    $options = array();
                    while ($fRow = $foreignTable->each())
                    {
                        $options[$fRow['id']] = substr($fRow['id'] . " (" . $fRow[$fField], 0, $form->size - 4) . ")";
                    }
                    $help = 'Foreign Key (<a class="override" href="' . $this->SELF . '?method=change';
                    $help .= '&table=' . $table . '&id=' . $id . '&override=true">override</a>)';
                    $changedRow[$field] = $form->select($label, $help, $options, $value);
                    $foreignTable->close();
                    continue;
                }
            }

            switch ($this->types[$field])
            {
                case "bool":
                    //$changedRow[$field] = $form->select($label, $help, array('1' => "true", '' => "false"), $value);
                    $changedRow[$field] = $form->checkbox($label, $help, array(1 => ""), $value);
                    break;
                case "int":
                case "float":
                    $changedRow[$field] = $form->text($label, $help, $value, 0, 10);
                    break;
                case "text":
                    $changedRow[$field] = $form->textarea($label, $help, $value, 7);
                    break;
                case "blob":
                    $file = $form->file($label, $help);
                    if ($file)
                    {
                        $fp = fopen($file['tmp_name'], "rb");
                        $changedRow[$field] = fread($fp, 10000);
                        fclose($fp);
                    }
                    break;
                default:
                    $changedRow[$field] = $form->text($label, $help, $value);
            }
        }
        // Show these both Buttons in edit/update() mode.
        $button = "Save Changes";
        if ($this->priv & 8) $button .= "|Delete";
        // Show this Button in insert() mode.
        if ($id == "") $button = "Insert new Row";
        $button = $form->submit($button);
        if ($form->isValid())
        {
            if ($button == "Delete") { $this->delete(); return; }
            if ($id != "") $this->table->update($changedRow, $id);
            else $this->table->insert($changedRow);
            $this->table->write();
            if ($id == "") { $this->browse(); return; }
        }

        $this->displayHead();

        $form->display();
    
        if ($id != "")
        {
            echo '<p>';
            $aHref = '<a href="' . $this->SELF . '?method=change&table=' . $table;
            if ($this->table->prev($id) === false)
                echo '&lt;&lt; First | &lt; Previous | ';
            else
            {
                echo $aHref . '&id=' . urlencode($this->table->first()) . '">&lt;&lt; First</a> | ';
                echo $aHref . '&id=' . urlencode($this->table->prev($id)) . '">&lt; Previous</a> | ';
            }
            if ($this->table->next($id) === false)
                echo 'Next &gt; | Last &gt;&gt;';
            else
            {
                echo $aHref . '&id=' . urlencode($this->table->next($id)) . '">Next &gt;</a> | ';
                echo $aHref . '&id=' . urlencode($this->table->last()) . '">Last &gt;&gt;</a>';
            }
            echo '</p>';
            if ($this->priv & 2)
                echo '<p>' . $aHref . '">Insert new Row</a></p>';
        }
    }

    /**
     * Shows a form to export the table.
     *
     * @return void
     */
    function export()
    {
        $form = new Apeform(0, 0, false);
        $form->hidden("export", "method");
        $table = $form->hidden($this->table->tablename(), "table");
        $options = array("csv" => "CSV/TXT", "sql" => "SQL dump", "xml" => "XML");
        $type = $form->radio("Export as", "", $options, "csv");
        $options = array("," => "Comma \",\"", ";" => "Semikolon \";\"", "\t" => "Tabulator \"\\t\"");
        $delimiter = $form->select("CSV/TXT Delimiter", "", $options, ",");
        $sections = $form->checkbox("SQL Sections", "", "Structure|Data", "Structure|Data");
        $save = $form->checkbox("Save as File");
        if (function_exists("gzencode")) $compress = $form->checkbox("Save gzip compressed");
        $form->submit("Export");
        if ($form->isValid() && $type == "csv")
        {
            $this->table->delimiter = $delimiter;
            $export = $this->table->export();
        }
        elseif ($form->isValid() && $type == "sql")
        {
            $tablename = preg_replace('/\W+/', '_', $this->table->tablename());
            $export = "CREATE TABLE " . $tablename . "(\n";
            foreach ($this->table->fields as $field)
            {
                $export .= "  " . preg_replace('/\W+/', '_', $field) . " ";
                if ($this->types[$field] == "varchar" || $this->types[$field] == "null") $export .= "varchar(255)";
                elseif ($this->types[$field] == "bool") $export .= "tinyint";
                else $export .= $this->types[$field];
                $export .= " NOT NULL";
                if ($field == "id") $export .= " auto_increment";
                $export .= ",\n";
            }
            $export .= "  PRIMARY KEY (id)\n);\n\n";
            if (! in_array("Structure", $sections)) $export = "";
            if (! in_array("Data", $sections)) $this->table->delete();
            while ($row = $this->table->each())
            {
                foreach ($row as $field => $value)
                {
                    $sqlRow[preg_replace('/\W+/', '_', $field)] = "'" . addslashes($value) . "'";
                }
                $export .= "INSERT INTO " . $tablename;
                $export .= " (" . implode(", ", array_keys($sqlRow)) . ") VALUES (" . implode(", ", $sqlRow) . ");\n";
            }
        }
        elseif ($form->isValid() && $type == "xml")
        {
            $tablename = preg_replace('/[^\w-]+/', '_', $this->table->tablename());
            $export = "<?xml version=\"1.0\"?>\n";
            $export .= "<" . $tablename . ">\n";
            while ($row = $this->table->each())
            {
                $export .= "    <row id=\"" . $row['id'] . "\">\n";
                unset($row['id']);
                foreach ($row as $field => $value)
                {
                    $field = preg_replace('/[^\w-]+/', '_', $field);
                    $export .= "        <" . $field . ">" . htmlspecialchars($value) . "</" . $field . ">\n";
                }
                $export .= "    </row>\n";
            }
            $export .= "</" . $tablename . ">\n";
        }

        if ($save)
        {
            $mimeType = 'text/comma-separated-values';
            $extension = "." . $type;
            if ($type == "csv" && $delimiter == "\t")
            {
                $mimeType = 'text/tab-separated-values';
                $extension = ".txt";
            }
            elseif ($type == "sql") $mimeType = 'application/octet-stream';
            elseif ($type == "xml") $mimeType = 'text/xml';
            if (isset($compress) && $compress)
            {
                $export = gzencode($export);
                $mimeType = 'application/x-gzip';
                $extension .= ".gz";
            }
            $filename = $this->table->tablename() . $extension;
            header('Content-Type: ' . $mimeType);
            header('Content-Size: ' . strlen($export));
            header('Content-Disposition: attachment; filename="' . $filename . '";');
            echo $export;
            return;
        }

        $this->displayHead();

        $form->display();

        echo "<pre>";
        if (isset($export)) echo htmlentities($export);
    }

    /**
     * Download binary file specified by tablename, id and field.
     *
     * @return void
     */
    function download()
    {
        $field = $this->GET('field');
        $file = $this->table->data[$this->GET('id')][$field];
        header('Content-Type: application/octet-stream');
        header('Content-Size: ' . strlen($file));
        header('Content-Disposition: attachment; filename="' . $field . '";');
        echo $file;
    }

    /**
     * Confirms the deletion of a row.
     *
     * @return void
     */
    function delete()
    {
        if ($this->GET('sure'))
        {
            $this->table->delete($this->GET('id'));
            $this->table->write();
            $this->browse();
            return;
        }
        $this->displayHead();
        echo 'Do you really want to<br>delete Row "' . $this->GET('id') . '"?<p>';
        echo '<a class="danger" href="' . $this->SELF . '?method=delete&table=' . $this->table->tablename() . '&id=' . $this->GET('id') . '&sure=yes">Yes</a> | ';
        echo '<a href="' . $this->SELF . '?method=browse&table=' . $this->table->tablename() . '">No</a>';
    }

    /**
     * Confirms the deletion of all rows.
     *
     * @return void
     */
    function delete_all()
    {
        if ($this->GET('sure'))
        {
            $this->table->delete();
            $this->table->write();
            $this->structure();
            return;
        }
        $this->displayHead();
        echo 'Do you really want to<br>delete all Rows from Table "' . $this->table->tablename() . '"?<p>';
        echo '<a class="danger" href="' . $this->SELF . '?method=delete_all&table=' . $this->table->tablename() . '&sure=yes">Yes</a> | ';
        echo '<a href="' . $this->SELF . '?method=browse&table=' . $this->table->tablename() . '">No</a>';
    }
    
    /**
     * Confirms the deletion of a column.
     *
     * @return void
     */
    function drop()
    {
        if ($this->GET('sure'))
        {
            $this->table->drop_field($this->GET('field'));
            $this->table->write();
            $this->structure();
            return;
        }
        $this->displayHead();
        echo 'Do you really want to<br>drop Field "' . $this->GET('field') . '"?<p>';
        echo '<a class="danger" href="' . $this->SELF . '?method=drop&table=' . $this->table->tablename() . '&field=' . $this->GET('field') . '&sure=yes">Yes</a> | ';
        echo '<a href="' . $this->SELF . '?method=structure&table=' . $this->table->tablename() . '">No</a>';
    }

    /**
     * Confirms the deletion of a table.
     *
     * @return void
     */
    function drop_table()
    {
        if ($this->GET('sure'))
        {
            $this->table->drop_table();
            $this->table->write();
            unset($this->table);
            $this->tables();
            return;
        }
        $this->displayHead();
        echo 'Do you really want to<br>drop Table "' . $this->table->tablename() . '"?<p>';
        echo '<a class="danger" href="' . $this->SELF . '?method=drop_table&table=' . $this->table->tablename() . '&sure=yes">Yes</a> | ';
        echo '<a href="' . $this->SELF . '?method=browse&table=' . $this->table->tablename() . '">No</a>';
    }
}

$admin = new MyCSVAdmin();
$method = $admin->GET('method');
if (! $method || ! method_exists($admin, $method)) $method = "tables";
$admin->$method();

?>